#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
#
# This script is licensed under GNU GPL version 2.0 or above
# (c) 2024 Antonio J. Delgado
# Based on  https://github.com/markuman/markuman.nextcloud/blob/latest/plugins/modules/password.py

"""Nextcloud Password client"""

from __future__ import absolute_import, division, print_function
__metaclass__ = type
import traceback
import json
import sys
import os
import logging
from logging.handlers import SysLogHandler
from xml.dom import minidom
import binascii
import click
import click_config_file
import passpy
try:
    import requests
    HAS_REQUESTS_LIB = True
except ImportError:
    HAS_REQUESTS_LIB = False
    IMPORT_ERROR = traceback.format_exc()

try:
    import pysodium
    HAS_PYSODIUM_LIB = True
except ImportError:
    HAS_PYSODIUM_LIB = False
    IMPORT_ERROR = traceback.format_exc()


DIOS_MIO = """<?xml version="1.0"?>
<d:propfind  xmlns:d="DAV:" xmlns:oc="http://owncloud.org/ns" xmlns:nc="http://nextcloud.org/ns">
<d:prop>
        <d:getlastmodified />
        <d:getetag />
        <d:getcontenttype />
        <d:resourcetype />
        <oc:fileid />
        <oc:permissions />
        <oc:size />
        <oc:checksums />
        <oc:favorite />
        <oc:owner-display-name />
        <oc:share-types />
</d:prop>
</d:propfind>
"""

class NextcloudErrorHandler:
    '''Handle errors in Nextcloud'''
    def __init__(self):
        self.config = {}
        self._init_log()

    def _init_log(self):
        ''' Initialize log object '''
        self._log = logging.getLogger("nc_password_client")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(
            logging.getLevelName(self.config.get("debug_level", 'INFO'))
        )
        self._log.addHandler(streamhandler)

        if 'log_file' in self.config:
            log_file = self.config['log_file']
        else:
            home_folder = os.environ.get(
                'HOME', os.environ.get('USERPROFILE', '')
            )
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "nc_password_client.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=102400000
        )
        # create formatter
        formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
        )
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True

    def status_code_error(self, status):
        '''Process status code error???'''
        if status > 399:
            self._log.error(
                'Nextcloud returned status code %s',
                status
            )
            sys.exit(1)
        else:
            self._log.debug(
                'Nextcloud returned status %s',
                status
            )

def parameter_spects(spec_arguments):
    '''Specification of parameters???'''
    argument_spec = dict(
        host=dict(required=False, type='str'),
        api_token=dict(required=False, type='str', no_log=True, aliases=['access_token']),
        user=dict(required=False, type='str'),
        ssl_mode=dict(required=False, type='str', default='https')
    )

    return {**argument_spec, **spec_arguments}


def decrypt(encrypted, key):
    '''Decrypt encrypted data'''
    nonce = encrypted[0:pysodium.crypto_secretbox_NONCEBYTES]
    ciphertext = encrypted[pysodium.crypto_secretbox_NONCEBYTES:]
    return pysodium.crypto_secretbox_open(ciphertext, nonce, key)


def decrypt_item(item_key, item, key='label'):
    '''Decrypt single item'''
    if len(item[key]) == 0:
        return ""
    vals = binascii.unhexlify(item[key])
    return decrypt(vals, binascii.unhexlify(item_key)).decode()


class NextcloudHandler:
    '''Handle Nextcloud Password API'''
    def __init__(self, params):
        self._init_log(params)
        self.exit = NextcloudErrorHandler()
        self.http = 'https'
        self.timeout = params.get('timeout', 3)
        self.ssl = True
        if params.get('ssl_mode') == 'http':
            self.http = 'http'
        elif params.get('ssl_mode') == 'skip':
            self.ssl = False
        elif os.environ.get('NEXTCLOUD_SSL_MODE') == 'http':
            self.http = 'http'
        elif os.environ.get('NEXTCLOUD_SSL_MODE') == 'skip':
            self.ssl = False

        self.details = params.get('details') or False
        self.x_api_session = None

        self.host = params.get('host') or os.environ.get('NEXTCLOUD_HOST')
        if self.host is None:
            self.exit.status_code_error('Unable to continue. No Nextcloud Host is given.')

        self.user = params.get('user') or os.environ.get('NEXTCLOUD_USER')
        if self.user is None:
            self.exit.status_code_error('Unable to continue. No Nextcloud User is given.')

        self.token = params.get('api_token') or os.environ.get('NEXTCLOUD_TOKEN')
        if self.token is None:
            self.exit.status_code_error('Unable to continue. No Nextcloud Token is given.')

        self.e2e_password = False
        if params.get('cse_password') or os.environ.get('NEXTCLOUD_CSE_PASSWORD'):
            # manage passwords client side encryption
            self.e2e_password = True
            password = params.get('cse_password') or os.environ.get('NEXTCLOUD_CSE_PASSWORD')
            retval = self.request_passwords_session()
            password_salt = binascii.unhexlify(retval['challenge']['salts'][0])
            generic_hash_key = binascii.unhexlify(retval['challenge']['salts'][1])
            password_hash_salt = binascii.unhexlify(retval['challenge']['salts'][2])

            generic_hash = pysodium.crypto_generichash(
                password.encode() + password_salt,
                k=generic_hash_key,
                outlen=pysodium.crypto_generichash_BYTES_MAX
            )
            password_hash = pysodium.crypto_pwhash(
                pysodium.crypto_box_SEEDBYTES,
                generic_hash,
                password_hash_salt,
                pysodium.crypto_pwhash_OPSLIMIT_INTERACTIVE,
                pysodium.crypto_pwhash_MEMLIMIT_INTERACTIVE,
                pysodium.crypto_pwhash_ALG_ARGON2ID13
            )
            # self.cse['keys']['CSEv1r1']
            self.cse = self.open_passwords_session(password_hash.hex())
            # decrypt CSEv1r1 chain
            vals = binascii.unhexlify(self.cse['keys']['CSEv1r1'])
            salt = vals[0:pysodium.crypto_pwhash_SALTBYTES]
            text = vals[pysodium.crypto_pwhash_SALTBYTES:]
            key = pysodium.crypto_pwhash(
                pysodium.crypto_box_SEEDBYTES,
                password,
                salt,
                pysodium.crypto_pwhash_OPSLIMIT_INTERACTIVE,
                pysodium.crypto_pwhash_MEMLIMIT_INTERACTIVE,
                pysodium.crypto_pwhash_ALG_ARGON2ID13
            )
            self.keychain = json.loads(decrypt(text, key))

    def _init_log(self, params):
        ''' Initialize log object '''
        self._log = logging.getLogger("nc_password_client")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(
            logging.getLevelName(params.get("debug_level", 'INFO'))
        )
        self._log.addHandler(streamhandler)

        if 'log_file' in params:
            log_file = params['log_file']
        else:
            home_folder = os.environ.get(
                'HOME', os.environ.get('USERPROFILE', '')
            )
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "nc_password_client.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=102400000
        )
        # create formatter
        formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
        )
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True

    def headers(self):
        '''Generate headers'''
        headers = {
            'Accept': 'application/json',
            'OCS-APIRequest': 'true'
        }
        if self.x_api_session:
            headers['X-API-SESSION'] = self.x_api_session
        return headers

    def get(self, path):
        '''Do a GET request'''
        r = requests.get(
            f'{self.http}://{self.host}/{path}',
            auth=(self.user, self.token), verify=self.ssl, headers=self.headers(),
            timeout=self.timeout
        )

        if r.status_code == 200:
            return r
        elif r.status_code == 404:
            self.exit.status_code_error(f'File {path} does not exist')
        else:
            self.exit.status_code_error(r.status_code)

    def propfind(self, path):
        '''Do a PROPFIND request'''
        s = requests.Session()
        s.auth = (self.user, self.token)
        r = s.request(
            method='PROPFIND',
            url=f'{self.http}://{self.host}/{path}',
            headers={'Depth': '0'},
            data=DIOS_MIO,
            verify=self.ssl
        )

        if r.status_code == 207:
            dom = minidom.parseString(r.text.encode('ascii', 'xmlcharrefreplace'))
            try:
                return {
                    'last_modified': dom.getElementsByTagName(
                        'd:getlastmodified'
                    )[0].firstChild.data,
                    'content_type': dom.getElementsByTagName(
                        'd:getcontenttype'
                    )[0].firstChild.data,
                    'file_id': int(dom.getElementsByTagName(
                        'oc:fileid'
                    )[0].firstChild.data),
                    'size': int(dom.getElementsByTagName(
                        'oc:size'
                    )[0].firstChild.data),
                    'favorite': int(dom.getElementsByTagName(
                        'oc:favorite'
                    )[0].firstChild.data),
                    'owner': dom.getElementsByTagName(
                        'oc:owner-display-name'
                    )[0].firstChild.data,
                    'href': dom.getElementsByTagName(
                        'd:href'
                    )[0].firstChild.data
                }
            except Exception:
                # I guess it's folder, because it has no content_type
                return {
                    'last_modified': dom.getElementsByTagName(
                        'd:getlastmodified'
                    )[0].firstChild.data,
                    'content_type': 'inode/directory',
                    'file_id': dom.getElementsByTagName(
                        'oc:fileid'
                    )[0].firstChild.data,
                    'size': dom.getElementsByTagName(
                        'oc:size'
                    )[0].firstChild.data,
                    'favorite': dom.getElementsByTagName(
                        'oc:favorite'
                    )[0].firstChild.data,
                    'owner': dom.getElementsByTagName(
                        'oc:owner-display-name'
                    )[0].firstChild.data,
                    'href': dom.getElementsByTagName(
                        'd:href'
                    )[0].firstChild.data
                }

        elif r.status_code == 404:
            return {}

        else:
            self.exit.status_code_error(r.status_code)

    def put(self, path, src=None):
        '''Do a PUT request'''
        if src:
            r = requests.put(
                f'{self.http}://{self.host}/{path}',
                data=open(src, 'rb'), auth=(self.user, self.token), verify=self.ssl, timeout=self.timeout
            )
        else:
            r = requests.put(
                f'{self.http}://{self.host}/{path}',
                headers=self.headers, auth=(self.user, self.token), verify=self.ssl, timeout=self.timeout
            )

        if r.status_code in [200, 201, 204]:
            return r, True
        else:
            self.exit.status_code_error(r.status_code)

    def delete(self, path):
        '''Do a DELETE request'''
        r = requests.delete(
            f'{self.http}://{self.host}/{path}',
            auth=(self.user, self.token), verify=self.ssl, timeout=self.timeout
        )

        if r.status_code in [200, 204]:
            return r, True
        elif r.status_code == 404:
            return r, False
        else:
            self.exit.status_code_error(r.status_code)

    def talk(self, message, channel):
        '''Post in Talk/Chat'''
        body = {
            'message': message,
            'replyTo': 0
        }

        spreed_v1_path = "ocs/v2.php/apps/spreed/api/v1/chat"

        r = requests.post(
            f'{self.http}://{self.host}/{spreed_v1_path}/{channel}',
            data=body,
            headers=self.headers(),
            auth=(self.user, self.token),
            verify=self.ssl, timeout=self.timeout
        )

        if r.status_code == 201:
            return r, True
        else:
            self.exit.status_code_error(r.status_code)

    def request_passwords_session(self):
        '''Request a Passwords API session'''
        r = self.get("index.php/apps/passwords/api/1.0/session/request")
        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def open_passwords_session(self, password_hash):
        '''Open a Passwords API session'''
        post_obj = {
            'challenge': password_hash
        }

        r = requests.post(
            f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/session/open',
            data=post_obj,
            headers=self.headers(),
            auth=(self.user, self.token),
            verify=self.ssl, timeout=self.timeout
        )
        if r.status_code == 200:
            self.x_api_session = r.headers.get('X-API-SESSION')
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def close_passwords_session(self):
        '''Close Passwords API session'''
        r = self.get("index.php/apps/passwords/api/1.0/session/close")
        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def list_passwords(self):
        '''List all passwords'''
        r = self.get("index.php/apps/passwords/api/1.0/password/list")
        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def list_passwords_folders(self):
        '''List passwords folders'''
        r = self.get("index.php/apps/passwords/api/1.0/folder/list")
        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def delete_passwords_folder(self, name):
        '''Delete a passwords folder'''
        all_folders = self.list_passwords_folders()
        folder_id = None
        for folder in all_folders:
            if folder['label'] == name:
                folder_id = folder['id']
                self._log.debug(
                    "Found folder with id '%s' to delete",
                    folder['id']
                )
        if folder_id:
            post_obj = {
                'id': folder_id
            }
            r = requests.delete(
                f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/folder/delete',
                data=post_obj,
                headers=self.headers(),
                auth=(self.user, self.token),
                verify=self.ssl,
                timeout=self.timeout
            )
            if r.status_code == 201:
                return r.json()
            else:
                self.exit.status_code_error(r.status_code)
        else:
            self._log.error(
                "Fodler '{name}' not found."
            )

    def exists_passwords_folder(self, name):
        '''Test if a passwords folder exists'''
        for folder in self.list_passwords_folders():
            if folder.get('label') == name:
                return True
        return False

    def create_passwords_folder(self, name):
        '''Create passwords folder'''
        if not self.exists_passwords_folder(name):
            post_obj = {
                'label': name
            }

            r = requests.post(
                f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/folder/create',
                data=post_obj,
                headers=self.headers(),
                auth=(self.user, self.token),
                verify=self.ssl,
                timeout=self.timeout
            )

            if r.status_code == 201:
                return r.json()
            else:
                self.exit.status_code_error(r.status_code)
        else:
            self._log.warning(
                "Password folder '%s' already exists",
                name
            )

    def get_passwords_folder(self, name):
        '''Get Passwords folder'''
        for folder in self.list_passwords_folders():
            if folder.get('label') == name:
                return folder.get('id')
        return None

    def get_password(self, name):
        '''Get a password'''
        r = self.list_passwords()
        ret = []
        for item in r:
            if item['cseType'] == 'CSEv1r1' and self.e2e_password:
                item_key = self.keychain['keys'].get(item['cseKey'])
                if item_key:
                    if decrypt_item(item_key, item, 'label') == name:
                        if self.details:
                            item['password'] = decrypt_item(item_key, item, 'password')
                            item['url'] = decrypt_item(item_key, item, 'url')
                            item['username'] = decrypt_item(item_key, item, 'username')
                            item['label'] = decrypt_item(item_key, item, 'label')
                            item['notes'] = decrypt_item(item_key, item, 'notes')
                            item['customFields'] = decrypt_item(item_key, item, 'customFields')
                            ret.append(item)
                        else:
                            ret.append(decrypt_item(item_key, item, 'password'))
            else:
                if item['label'] == name:
                    if self.details:
                        ret.append(item)
                    else:
                        ret.append(item['password'])
        return ret

    def fetch_generated_password(self):
        '''Fetch a generated password'''
        r = self.get('index.php/apps/passwords/api/1.0/service/password')
        if r.status_code == 200:
            return [r.json().get('password')]
        else:
            self.exit.status_code_error(r.status_code)

    def exists_password(self, name):
        for password in self.list_passwords():
            if password['label'] == name:
                return True
        return False

    def create_password(self, post_obj):
        '''Create/add a password'''
        if not self.exists_password(post_obj['label']):
            r = requests.post(
                f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/password/create',
                data=post_obj,
                headers=self.headers(),
                auth=(self.user, self.token),
                verify=self.ssl, timeout=self.timeout
            )

            if r.status_code == 201:
                return r.json()
            else:
                self._log.error(r.json())
                self.exit.status_code_error(r.status_code)
        else:
            self._log.warning(
                "Password with name '%s' already exists",
                post_obj['label']
            )

    def delete_password(self, post_obj):
        '''Delete a password'''
        r = requests.delete(
            f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/password/delete',
            data=post_obj,
            headers=self.headers(),
            auth=(self.user, self.token),
            verify=self.ssl, timeout=self.timeout
        )

        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

    def update_password(self, post_obj):
        '''Update a password'''
        r = requests.patch(
            f'{self.http}://{self.host}/index.php/apps/passwords/api/1.0/password/update',
            data=post_obj,
            headers=self.headers(),
            auth=(self.user, self.token),
            verify=self.ssl, timeout=self.timeout
        )

        if r.status_code == 200:
            return r.json()
        else:
            self.exit.status_code_error(r.status_code)

class NcPasswordClient:
    '''Nextcloud Password Client'''

    def __init__(self, debug_level, log_file, host, user, api_token, cse_password, timeout):
        self.config = {}
        self.config['debug_level'] = debug_level
        if log_file is None:
            self.config['log_file'] = os.path.join(
                os.environ.get(
                    'HOME',
                    os.environ.get(
                        'USERPROFILE',
                        os.getcwd()
                    )
                ),
                'log',
                'nc_password_client.log'
            )
        self._init_log()
        params = {
            "host": host,
            "user": user,
            "api_token": api_token,
            "cse_password": cse_password,
            "timeout": timeout,
        }
        self.nc = NextcloudHandler(params)

    def get_password(self, name):
        '''Get a single password'''
        print(json.dumps(self.nc.get_password(name), indent=2))

    def list_passwords(self):
        '''List all passwords'''    
        print(json.dumps(self.nc.list_passwords(), indent=2))

    def list_passwords_folders(self):
        '''List all password folders'''    
        print(json.dumps(self.nc.list_passwords_folders(), indent=2))

    def create_password(self, obj):
        '''Create a password with an object'''
        self._log.debug(
            "Creating password %s",
            obj
        )
        print(json.dumps(self.nc.create_password(obj), indent=2))

    def delete_password(self, name):
        '''Delete a password'''
        for password in self.nc.list_passwords():
            if password['label'] == name:
                self._log.debug(
                    "Deleting password:%s",
                    json.dumps(password)
                )
                print(json.dumps(self.nc.delete_password(password), indent=2))
                return True
        self._log.warning(
            "Password with name '%s' doesn't exist",
            name
        )

    def create_passwords_folder(self, name):
        '''Create a passwords folder'''
        print(json.dumps(self.nc.create_passwords_folder(name), indent=2))

    def delete_passwords_folder(self, name):
        '''Delete a passwords folder'''
        print(json.dumps(self.nc.delete_passwords_folder(name), indent=2))

    def migrate_pass(self):
        '''Migrate password store to Nextcloud pass'''
        store = passpy.store.Store()
        for item in store.find(''):
            obj = {
                "label": os.path.basename(item),
            }
            folder = os.path.dirname(item)
            if folder != '':
                obj['folder'] = folder
            raw_values = store.get_key(item).split('\n')
            obj['password'] = raw_values[0]
            raw_values.pop(0)
            for line in raw_values:
                if ': ' in line:
                    split_line = line.split(': ')
                    field = split_line[0]
                    value = split_line[1]
                else:
                    field = line
                    value = ''
                if field != '' and value != '':
                    obj[field] = value
            print(json.dumps(self.nc.create_password(obj), indent=2))

    def _init_log(self):
        ''' Initialize log object '''
        self._log = logging.getLogger("nc_password_client")
        self._log.setLevel(logging.DEBUG)

        sysloghandler = SysLogHandler()
        sysloghandler.setLevel(logging.DEBUG)
        self._log.addHandler(sysloghandler)

        streamhandler = logging.StreamHandler(sys.stdout)
        streamhandler.setLevel(
            logging.getLevelName(self.config.get("debug_level", 'INFO'))
        )
        self._log.addHandler(streamhandler)

        if 'log_file' in self.config:
            log_file = self.config['log_file']
        else:
            home_folder = os.environ.get(
                'HOME', os.environ.get('USERPROFILE', '')
            )
            log_folder = os.path.join(home_folder, "log")
            log_file = os.path.join(log_folder, "nc_password_client.log")

        if not os.path.exists(os.path.dirname(log_file)):
            os.mkdir(os.path.dirname(log_file))

        filehandler = logging.handlers.RotatingFileHandler(
            log_file, maxBytes=102400000
        )
        # create formatter
        formatter = logging.Formatter(
            '%(asctime)s %(name)-12s %(levelname)-8s %(message)s'
        )
        filehandler.setFormatter(formatter)
        filehandler.setLevel(logging.DEBUG)
        self._log.addHandler(filehandler)
        return True


@click.group()
@click.option(
    "--debug-level",
    "-d",
    default="INFO",
    type=click.Choice(
        ["CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG", "NOTSET"],
        case_sensitive=False,
    ),
    help='Set the debug level for the standard output.'
)
@click.option('--log-file', '-l', help="File to store all debug messages.")
# @click.option("--dummy","-n", is_flag=True,
# help="Don't do anything, just show what would be done.")
@click.option(
    '--host', '-H',
    required=True,
    help='Nextcloud host name'
)
@click.option(
    '--user', '-u',
    required=True,
    help='Nextcloud user name'
)
@click.option(
    '--api-token', '-a',
    required=True,
    help='Nextcloud user\'s application token'
)
@click.option(
    '--cse-password', '-p',
    help='Nextcloud user\'s end-to-end encryption password'
)
@click.option(
    '--timeout', '-t',
    default=3,
    type=click.INT,
    help='Nextcloud user\'s end-to-end encryption password'
)
@click_config_file.configuration_option()
@click.pass_context
def cli(ctx, debug_level, log_file, host, user, api_token, cse_password, timeout):
    '''Client function to pass context'''
    ctx.ensure_object(dict)
    ctx.obj['NcPasswordClient'] = NcPasswordClient(
        debug_level, log_file, host, user, api_token, cse_password, timeout
    )

@cli.command()
@click.option('--name', '-n', required=True, help='Name of the password to show')
@click_config_file.configuration_option()
@click.pass_context
def show(ctx, name):
    '''Show a single password'''
    ctx.obj['NcPasswordClient'].get_password(name)

@cli.command()
@click_config_file.configuration_option()
@click.pass_context
def ls(ctx):
    '''Show all password'''
    ctx.obj['NcPasswordClient'].list_passwords()

@cli.command()
@click.option('--obj', '-o', required=True, help='JSON object for a password')
@click_config_file.configuration_option()
@click.pass_context
def create_password(ctx, obj):
    '''Create a password'''
    ctx.obj['NcPasswordClient'].create_password(json.loads(obj))

@cli.command()
@click.option('--name', '-n', required=True, help='Name of the password to delete')
@click_config_file.configuration_option()
@click.pass_context
def delete_password(ctx, name):
    '''Delete a password'''
    ctx.obj['NcPasswordClient'].delete_password(name)

@cli.command()
@click.option('--name', '-n', required=True, help='Name of the passwords folder to create')
@click_config_file.configuration_option()
@click.pass_context
def create_passwords_folder(ctx, name):
    '''Create a password folder'''
    ctx.obj['NcPasswordClient'].create_passwords_folder(name)

@cli.command()
@click_config_file.configuration_option()
@click.pass_context
def list_passwords_folders(ctx):
    '''List all password folders'''
    ctx.obj['NcPasswordClient'].list_passwords_folders()

@cli.command()
@click.option('--name', '-n', required=True, help='Name of the passwords folder to delete')
@click_config_file.configuration_option()
@click.pass_context
def delete_passwords_folder(ctx, name):
    '''Delete a password folder'''
    ctx.obj['NcPasswordClient'].delete_passwords_folder(name)

@cli.command()
@click_config_file.configuration_option()
@click.pass_context
def migrate_pass(ctx):
    '''Migrate password store passwords to Nextcloud Passwords'''
    ctx.obj['NcPasswordClient'].migrate_pass()

if __name__ == "__main__":
    cli(obj={})
